from django.db import models

from oscar.apps.shipping.abstract_models import AbstractOrderAndItemLevelChargeMethod


class OrderAndItemLevelChargeMethod(AbstractOrderAndItemLevelChargeMethod):
    pass