<!DOCTYPE html> 
<html lang="{% block language %}en-gb{% endblock %}">
    <head>
        <title>{% block title %}Oscar :: Flexible ecommerce for Django{% endblock %}</title>
        <meta http-equiv="content-type" content="text/html; charset=UTF-8"/>
        <meta name="created" content='{% now "jS M Y h:i" %}' />
        <meta name="description" content="{% block description %}{% endblock %}" />
        <meta name="keywords" content="{% block keywords %}{% endblock %}" />
        <link rel="stylesheet" href="{{ STATIC_URL }}css/base/screen.css" />
        {% block extra_head %}{% endblock %}
    </head>
    <body id="{% block body_id %}default{% endblock %}" class="{% block body_class %}default{% endblock %}">
        {% block layout %}{% endblock %}
        {% block tracking %}
            {% if not debug and not request.user.is_staff %}
                <!-- Tracking to go here. -->
            {% endif %}
        {% endblock %}
    </body>
</html>
