from oscar.apps.discount.abstract_models import (AbstractDiscountOffer, PERCENTAGE_DISCOUNT, 
                                            ABSOLUTE_DISCOUNT, FINAL_PRICE)

class DiscountOffer(AbstractDiscountOffer):
    pass

