{% extends "oscar/checkout/checkout.html" %}

{% load currency_filters %}

{% block payment_method %}

<h3>Payment</h3>
<hr />
<table>
    <thead>
        <tr>
            <th>Method</th>
            <th></th>
        </tr>
    <thead>
    <tbody>
        <tr>
            <td>Cheque</td>
            <td>
                <form method="post" action="{% url oscar-checkout-payment-method %}">
                    {% csrf_token %}
                    <input type="hidden" name="method_code" value="cheque" />
                    <input type="submit" value="Pay by cheque" />
                </form>
            </td>
        </tr>
        <tr>
            <td>Bankcard</td>
            <td>
                <form method="post" action="{% url oscar-checkout-payment-method %}">
                    {% csrf_token %}
                    <input type="hidden" name="method_code" value="bankcard" />
                    <input type="submit" value="Pay by bankcard" />
                </form>
            </td>
        </tr>
    </tbody>
</table>

{% endblock payment_method %}
