from oscar.apps.address.tests import *
from oscar.apps.basket.tests import *
from oscar.apps.order.tests import *
from oscar.apps.catalogue.tests import *
from oscar.apps.partner.tests import *
from oscar.apps.checkout.tests import *
from oscar.apps.payment.tests import *
from oscar.apps.offer.tests import *
from oscar.apps.shipping.tests import *
from oscar.apps.customer.tests import *
from oscar.apps.discount.tests import *
from oscar.apps.promotions.tests import *
from oscar.apps.catalogue.reviews.tests import *
from oscar.apps.voucher.tests import *
from oscar.apps.partner.tests import *

from oscar.core.tests import *
from oscar.core.logging.tests import *
